@@ -4,9 +4,9 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
+#include "AbstractReviewsBackend.h"
 #include <KConfigGroup>
 #include <KSharedConfig>
-#include "AbstractReviewsBackend.h"
 
 AbstractReviewsBackend::AbstractReviewsBackend(QObject *parent)
     : QObject(parent)
@@ -35,7 +35,11 @@ QString AbstractReviewsBackend::preferredUserName() const
     }
 }
 
-void AbstractReviewsBackend::submitReview(AbstractResource *app, const QString &summary, const QString &review_text, const QString &rating, const QString &userName)
+void AbstractReviewsBackend::submitReview(AbstractResource *app,
+                                          const QString &summary,
+                                          const QString &review_text,
+                                          const QString &rating,
+                                          const QString &userName)
 {
     if (supportsNameChange() && !userName.isEmpty()) {
         auto config = KSharedConfig::openConfig();
