@@ -519,6 +519,10 @@ FlatpakResource *FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *inst
         }
     }
 
+    if (!source) {
+        return nullptr;
+    }
+
     FlatpakResource *resource = new FlatpakResource(cid, source->installation(), const_cast<FlatpakBackend *>(this));
     resource->setOrigin(source->name());
     resource->setDisplayOrigin(source->title());
@@ -661,7 +665,10 @@ void FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url, ResultsStream *str
         FlatpakRef *ref = FLATPAK_REF(g_ptr_array_index(refs, i));
         FlatpakInstalledRef *iref = FLATPAK_INSTALLED_REF(g_ptr_array_index(refs, i));
         if (qstrcmp(flatpak_ref_get_commit(ref), flatpak_ref_get_commit(FLATPAK_REF(bundleRef))) == 0) {
-            Q_EMIT stream->resourcesFound({getAppForInstalledRef(preferredInstallation(), iref, nullptr)});
+            auto res = getAppForInstalledRef(preferredInstallation(), iref, nullptr);
+            if (res) {
+                Q_EMIT stream->resourcesFound({res});
+            }
             return;
         }
     }
@@ -1003,13 +1010,9 @@ void FlatpakBackend::loadRemote(FlatpakInstallation *installation, FlatpakRemote
 
     g_autofree char *path_str = g_file_get_path(fileTimestamp);
     QFileInfo fileInfo(QFile::encodeName(path_str));
-    if (!fileInfo.exists()) {
-        checkForRemoteUpdates(installation, remote);
-
+    if (!fileInfo.exists() || fileInfo.lastModified().toUTC().secsTo(QDateTime::currentDateTimeUtc()) > 21600) {
         // Refresh appstream metadata in case they have never been refreshed or the cache is older than 6 hours
-        if (fileInfo.lastModified().toUTC().secsTo(QDateTime::currentDateTimeUtc()) > 21600) {
-            connect(this, &FlatpakBackend::initialized, m_checkForUpdatesTimer, qOverload<>(&QTimer::start));
-        }
+        checkForRemoteUpdates(installation, remote);
     } else {
         auto source = integrateRemote(installation, remote);
         Q_ASSERT(findSource(installation, flatpak_remote_get_name(remote)) == source);
@@ -1440,12 +1443,14 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                         bool fresh;
                         auto resource = getAppForInstalledRef(it.key(), ref, &fresh);
                         g_object_unref(ref);
-                        resource->setState(AbstractResource::Upgradeable, !fresh);
-                        updateAppSize(resource);
-                        if (resource->resourceType() == FlatpakResource::Runtime) {
-                            resources.prepend(resource);
-                        } else {
-                            resources.append(resource);
+                        if (resource) {
+                            resource->setState(AbstractResource::Upgradeable, !fresh);
+                            updateAppSize(resource);
+                            if (resource->resourceType() == FlatpakResource::Runtime) {
+                                resources.prepend(resource);
+                            } else {
+                                resources.append(resource);
+                            }
                         }
                     }
                 }
@@ -1521,6 +1526,9 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                         continue;
 
                     auto resource = getAppForInstalledRef(installation, ref);
+                    if (!resource) {
+                        continue;
+                    }
                     if (!filter.search.isEmpty() && !resource->name().contains(filter.search, Qt::CaseInsensitive))
                         continue;
 
@@ -1821,6 +1829,7 @@ Transaction *FlatpakBackend::removeApplication(AbstractResource *app)
 
 void FlatpakBackend::checkForUpdates()
 {
+    disconnect(this, &FlatpakBackend::initialized, m_checkForUpdatesTimer, qOverload<>(&QTimer::start));
     for (const auto &source : qAsConst(m_flatpakSources)) {
         if (source->remote()) {
             Q_ASSERT(!m_refreshAppstreamMetadataJobs.contains(source->remote()));
