@@ -12,6 +12,7 @@
 
 #include <AppStreamQt/release.h>
 #include <AppStreamQt/utils.h>
+#include <Category/Category.h>
 #include <KLocalizedString>
 #include <appstream/AppStreamIntegration.h>
 
@@ -27,6 +28,7 @@ static const QString DevelopmentVersionName = QStringLiteral("Rawhide");
 RpmOstreeBackend::RpmOstreeBackend(QObject *parent)
     : AbstractResourcesBackend(parent)
     , m_currentlyBootedDeployment(nullptr)
+    , m_transaction(nullptr)
     , m_watcher(new QDBusServiceWatcher(this))
     , m_interface(nullptr)
     , m_updater(new StandardBackendUpdater(this))
@@ -142,13 +144,8 @@ void RpmOstreeBackend::initializeBackend()
 
     // Look for a potentially already in-progress rpm-ostree transaction that
     // was started outside of Discover
-    const QString transaction = m_interface->activeTransactionPath();
-    if (!transaction.isEmpty()) {
-        qInfo() << "rpm-ostree-backend: A transaction is already in progress";
-        // We don't check that m_currentlyBootedDeployment is != nullptr here as we've
-        // initialized it right above in refreshDeployments.
-        m_transaction = new RpmOstreeTransaction(this, m_currentlyBootedDeployment, m_interface, RpmOstreeTransaction::Unknown);
-        TransactionModel::global()->addTransaction(m_transaction);
+    if (hasExternalTransaction()) {
+        setFetching(false);
         return;
     }
 
@@ -159,6 +156,8 @@ void RpmOstreeBackend::initializeBackend()
 
 void RpmOstreeBackend::refreshDeployments()
 {
+    // Set fetching in all cases but record the state to decide if we should undo it at the end
+    bool wasFetching = isFetching();
     setFetching(true);
 
     // Get the path for the curently booted OS DBus interface.
@@ -199,7 +198,55 @@ void RpmOstreeBackend::refreshDeployments()
     // The number of updates might have changed if we're called after an update
     Q_EMIT updatesCountChanged();
 
-    setFetching(false);
+    // Only undo the fetching state if we set it up in this function
+    if (!wasFetching) {
+        setFetching(false);
+    }
+}
+
+void RpmOstreeBackend::transactionStatusChanged(Transaction::Status status)
+{
+    switch (status) {
+    case Transaction::Status::DoneStatus:
+    case Transaction::Status::DoneWithErrorStatus:
+    case Transaction::Status::CancelledStatus:
+        m_transaction = nullptr;
+        setFetching(false);
+        break;
+    default:
+        // Ignore all other status changes
+        ;
+    }
+}
+
+void RpmOstreeBackend::setupTransaction(RpmOstreeTransaction::Operation op, QString arg)
+{
+    m_transaction = new RpmOstreeTransaction(this, m_currentlyBootedDeployment, m_interface, op, arg);
+    connect(m_transaction, &RpmOstreeTransaction::statusChanged, this, &RpmOstreeBackend::transactionStatusChanged);
+    connect(m_transaction, &RpmOstreeTransaction::deploymentsUpdated, this, &RpmOstreeBackend::refreshDeployments);
+    connect(m_transaction, &RpmOstreeTransaction::lookForNextMajorVersion, this, &RpmOstreeBackend::lookForNextMajorVersion);
+}
+
+bool RpmOstreeBackend::hasExternalTransaction()
+{
+    // Do we already know that we have a transaction in progress?
+    if (m_transaction) {
+        qInfo() << "rpm-ostree-backend: A transaction is already in progress";
+        return true;
+    }
+
+    // Is there actualy a transaction in progress we don't know about yet?
+    const QString transaction = m_interface->activeTransactionPath();
+    if (!transaction.isEmpty()) {
+        qInfo() << "rpm-ostree-backend: Found a transaction in progress";
+        // We don't check that m_currentlyBootedDeployment is != nullptr here as we expect
+        // that the backend is initialized when we're called.
+        setupTransaction(RpmOstreeTransaction::Unknown);
+        TransactionModel::global()->addTransaction(m_transaction);
+        return true;
+    }
+
+    return false;
 }
 
 void RpmOstreeBackend::checkForUpdates()
@@ -209,7 +256,16 @@ void RpmOstreeBackend::checkForUpdates()
         return;
     }
 
-    m_transaction = new RpmOstreeTransaction(this, m_currentlyBootedDeployment, m_interface, RpmOstreeTransaction::CheckForUpdate);
+    // Do not start a transaction if there is already one in-progress (likely externaly started)
+    if (hasExternalTransaction()) {
+        qInfo() << "rpm-ostree-backend: Not checking for updates while a transaction is in progress";
+        return;
+    }
+
+    // We're fetching updates
+    setFetching(true);
+
+    setupTransaction(RpmOstreeTransaction::CheckForUpdate);
     connect(m_transaction, &RpmOstreeTransaction::newVersionFound, [this](QString newVersion) {
         // Mark that there is a newer version for the current deployment
         m_currentlyBootedDeployment->setNewVersion(newVersion);
@@ -239,7 +295,6 @@ void RpmOstreeBackend::checkForUpdates()
             Q_EMIT inlineMessageChanged(m_rebootBeforeRebaseMessage);
         }
     });
-    connect(m_transaction, &RpmOstreeTransaction::lookForNextMajorVersion, this, &RpmOstreeBackend::lookForNextMajorVersion);
     m_transaction->start();
     TransactionModel::global()->addTransaction(m_transaction);
 }
@@ -420,6 +475,11 @@ bool RpmOstreeBackend::isValid() const
 
 ResultsStream *RpmOstreeBackend::search(const AbstractResourcesBackend::Filters &filter)
 {
+    // Only return results when we're explicitely looking in the "Operating System" category
+    if (filter.category && filter.category->untranslatedName() != QLatin1String("Operating System")) {
+        return new ResultsStream(QStringLiteral("rpm-ostree-empty"), {});
+    }
+
     QVector<AbstractResource *> res;
     for (RpmOstreeResource *r : m_resources) {
         if (r->state() >= filter.state) {
@@ -448,8 +508,7 @@ Transaction *RpmOstreeBackend::installApplication(AbstractResource *app)
         return nullptr;
     }
 
-    m_transaction = new RpmOstreeTransaction(this, m_currentlyBootedDeployment, m_interface, RpmOstreeTransaction::Update);
-    connect(m_transaction, &RpmOstreeTransaction::deploymentsUpdated, this, &RpmOstreeBackend::refreshDeployments);
+    setupTransaction(RpmOstreeTransaction::Update);
     m_transaction->start();
     return m_transaction;
 }
@@ -488,9 +547,7 @@ void RpmOstreeBackend::rebaseToNewVersion()
 
     // Only start one rebase operation at a time
     Q_EMIT inlineMessageChanged(nullptr);
-    m_transaction = new RpmOstreeTransaction(this, m_currentlyBootedDeployment, m_interface, RpmOstreeTransaction::Rebase, ref);
-    connect(m_transaction, &RpmOstreeTransaction::deploymentsUpdated, this, &RpmOstreeBackend::refreshDeployments);
-    connect(m_transaction, &RpmOstreeTransaction::lookForNextMajorVersion, this, &RpmOstreeBackend::lookForNextMajorVersion);
+    setupTransaction(RpmOstreeTransaction::Rebase, ref);
     m_transaction->start();
     TransactionModel::global()->addTransaction(m_transaction);
 }
